      module netCDF_file_read
! ***************************************************************************   
! This module contains routines to read in a netcdf file whose name is lnetCDF_file
! The routines no nothing about the structure of the vgosDB,or where they go in the solve data structure.
! This later information is contained in vgosDB_session_read_mod
!
      implicit none
      include 'netcdf.inc'
      include 'vgosDB_err.inc'      
 
      character*128 lnetCDF_file
      integer ncid                  !handle.
      integer iretval               !return value from calls 


! Function definitions.
      integer idim_size(4)                     !number of dimensions of array 
      integer num_dim                          !number of dimensions
! Generic NetCDF variables used to open vgosDB files and get information.
 
      character*10 lvgosDB_type(6)/"Byte","String","Integer*2","Integer*4","Real*4","Real*8"/
! Dimension ID
      integer max_dimid,num_dimid
      parameter (max_dimid=50)
      character*30 ldim_name(max_dimid),ldim_name_cap(max_dimid)
      integer idimid
      integer idim_size_var(max_dimid)
      integer num_gatts

! Variable ID
      integer num_varid
      integer, parameter :: max_Varid=500
      character*30 lvar_name(max_varid),lvar_name_cap(max_varid)
      integer num_dim_var(max_varid)
      integer max_dim_per_var
      parameter (max_dim_per_var=5)
      integer ivar_dimid(max_dim_per_var,max_varid)   !these are variable ids for each dimension.
      integer ivar_repeat(max_varid)                  !how many times the values are repeated (if any) 
      integer num_atts
      integer ivarid 
      integer ivar_type(max_varid)
! Miscellaneous
      integer unlimdimid 

      contains
! ************************************
      subroutine read_netCDF_var_info(kverbose,lerr_msg,ierr)
! passsed
      logical kverbose
! returned
      character*(*)  lerr_msg
      integer        ierr
      integer j      !counter

! Open a vgosDB file in NetCDF format in READ_ONLY access and get information from the header:
!   Number of dimensions, number of variables, types, etc. 
!   The file is left open upon exit and must be closed.
!
       ierr=0 
! Open the file
       iretval=nf_open(lnetCDF_file,NF_NOWRITE,ncid)
       if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)

! inquire what is in the file
       iretval=nf_inq(ncid, num_dimid, num_varid, num_gatts, unlimdimid)
       if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
       if(kverbose) then
         write(*,'("NCID = ",i4," NumDims = ", i4, " NumVars ", i4, " NumAtt = ", i4)') &
      &   ncid, num_dimid, num_varid, num_gatts 
        endif

       if(Num_Dimid .gt. Max_Dimid) then
         write(*,*) "vgosDB_Mod: Out of space! Please increase Max_Dimid to > ", Num_Dimid
       endif 

       if(kverbose) then
         write(*,'(A)') " Dimensions     "
         write(*,'(A)') "  # Name                   Size"
       endif
       ldim_name(1:Num_dimid)= " "
       do idimid=1,Num_dimid
         iretval=nf_inq_dim(ncid, idimid,ldim_name(idimid), idim_size_var(idimid))
         if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
         ldim_name_cap(idimid)=ldim_name(idimid)
         call capitalize(ldim_name_cap(idimid))
         if(kverbose) then
           write(*,'(i3,1x,a20,1x,i6)') idimid, ldim_name(idimid), idim_size_var(idimid)
          endif
       end do 

       if(kverbose) then
         write(*,'(A)') "Variables" 
         write(*,'(A)') "  # Name                 Type        Dimensions "
       endif
       
       if(Num_Varid .gt. Max_Varid) then
         write(*,*) "vgosDB_Mod: Out of space! Please increase Max_Varid to > ", Num_Varid
       endif 

       lvar_name(1:Num_varid)= " " 
       do ivarid=1,Num_Varid
         iretval=nf_inq_var(ncid,ivarid,lvar_name(ivarid),ivar_type(ivarid),   &
     &               num_dim_var(ivarid),ivar_dimid(1,ivarid), num_atts)
         if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)         

         lvar_name_cap(ivarid)=lvar_name(ivarid)
         call capitalize(lvar_name_cap(ivarid))

         if(num_dim_var(ivarid) .gt. max_dim_per_var) then
            write(*,*) "vgosDB_RW_Mod: Two many dimensions for: ",lvar_name(ivarid)
            stop
         endif
         
! Check to see if there is the "repeat" attribute. If so find how many repetitions.         
         iretval=nf_get_att_int(ncid,ivarid,"REPEAT",ivar_repeat(ivarid))
         if(iretval .ne. 0) then
            ivar_repeat(ivarid)=0
          else          
!            write(*,*) "Repeat-->",lvar_name(ivarid), ivar_repeat(ivarid) 
          endif          

         if(kverbose) then
           write(*,'(i3,1x,a20,1x,a10," | ", 3(1x,i7))') ivarid, lvar_name(ivarid), &
     &       lvgosDB_type(ivar_type(ivarid)), (idim_size_var(ivar_dimid(j,ivarid)),j=1,num_dim_var(ivarid))    
         endif
       end do 
      return
      end subroutine
     
! ************************************************************************************
      subroutine get_netCDF_dim(ldim_name_want,idim_size_want,lerr_msg,ierr)
! check if we have a given dimension (upto case). If so return it. If not, die.
      character*(*) ldim_name_want
 ! Returned
      integer idim_size_want
      character*(*) lerr_msg   
      integer ierr                       !<>0 some error
       
! function
      integer iwhere_in_string_list
! local
      character*(len(ldim_name_want)) ldim_name_want_cap
  
      ierr=0
      ldim_name_want_cap=ldim_name_want
      call capitalize(ldim_name_want_cap)

!      write(*,*) "Searching for dimension ", ldim_name_want_cap    
!      write(*,*) ldim_name_cap(1:num_dimid) 
      idimid=iwhere_in_string_list(ldim_name_cap,num_dimid,ldim_name_want_cap)
      if(idimid .eq. 0) then
        ierr=ivgosDB_dimid_not_found
        lerr_msg="Missing DIMID "//Trim(ldim_name_want)//" from file "//TRIM(lnetCDF_file)
        return
       endif
       idim_size_want=idim_size_var(idimid)
       
       return
       end subroutine
! *******************************************************************
      subroutine get_netCDF_varid(lvar_name_want,ivarid,lerr_msg,ierr)
!  Get ther varid for vgosDB variable
! passed
      character*(*) lvar_name_want
 ! returned
      integer ivarid
      character*(*) lerr_msg
      integer ierr

! function
      integer iwhere_in_string_list
! local
      character*(len(lvar_name_want)) lvar_name_want_cap

      ierr=0
      lvar_name_want_cap=lvar_name_want
      call capitalize(lvar_name_want_cap) 
  
      ivarid=iwhere_in_string_list(lvar_name_cap,num_varid,lvar_name_want_cap)
      if(ivarid .eq. 0) then
        ierr=ivgosDB_varid_not_found 
!        write(*,*) lvar_name_cap(1:num_varid)
        lerr_msg="Missing VARID "//Trim(lvar_name_want)//" from file "//trim(lnetCDF_file)
!        stop
!        write(*,*) trim(lerr_msg)
!        write(*,*) lvar_name_cap(1:num_Varid)
        return
      endif
      return
      end subroutine

! ********************************************************************
      subroutine get_real8_value_or_set_default(lvar_want, default, rvalue)
! passed
      character*(*) lvar_want
      real*8 default
      real*8 rvalue
! Functions
      integer iwhere_in_string_list 
! local
      character*30 lvar_want_cap
      integer ivarid
    
      if(len(lvar_want) .gt. len(lvar_want_cap)) then
         writE(*,*) "get_value_or_set_default: Recompile increasing size to at least ", len(lvar_want)
         stop
      endif
      lvar_want_cap=lvar_want
      call capitalize(lvar_want_cap)
      ivarid=iwhere_in_string_list(lvar_name_cap,num_varid,lvar_want_cap)
      if(ivarid .eq. 0) then
        rvalue=default
      else      
        iretval=nf_get_var_double(ncid,ivarid,rvalue)
        if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)          
      endif 

      return
      end subroutine 

! *************************************************************************
      subroutine get_netCDF_double_arraynames(lNameList,NumNames,lerr_msg,ierr)
! Get a list or real*8 arraynames, 
      character*(*) lnamelist(*)
      integer NumNames
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error

      ierr=0
      NumNames=0
      do ivarid=1,Num_Varid
        if(ivar_type(ivarid) .eq. nf_double)  then  !real*8
           NumNames=NumNames+1
           lNameList(NumNames)=lvar_name(ivarid)
        endif
      end do 
      return

      end subroutine 

! ****************************************************************************************
      subroutine get_netCDF_string(lvar_name,lstring,lerr_msg,ierr)
! Read in a string from vgosDB. (NOT a string array)
! Exit with error if a problem.
      character*(*) lvar_name          !Name of string we want to read
      character*(*) lstring            !place where it is stored.
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error       
! local
      integer ilen_string,ilen_get
      integer istart/1/
     
      lstring=" "
! Start of code 
      call get_netCDF_varid(lvar_name,ivarid,lerr_msg,ierr)
      if(ierr .ne. 0) return

      ilen_string=len(lstring)
      ilen_get=idim_size_var(ivar_dimid(1,ivarid)) 

      if(num_dim_var(ivarid) .gt. 1) then      
        ierr=ivgosDB_number_dimensions_mismatch
        lerr_msg="1. Expecting scalar value for string '"//trim(lvar_name)//"' but got array"
        return
      else if(ilen_get .gt. ilen_string) then   
         write(*,'("get_netCDF_string: Not enough space to store value of string ",a,&
     &              " need ",i4, " only have ", i4, " truncating!")') &
     &   trim(lvar_name), idim_size_var(ivar_dimid(1,ivarid)), len(lstring)
        ilen_get=ilen_string   
      endif
      
! read in the string.
  
      iretval=nf_get_vara_text(ncid,ivarid,istart,ilen_get,lstring)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)

      return
      end subroutine  

! ****************************************************************************************
      subroutine get_netCDF_string_1d_alloc(lvar_name,lstring,ilen,kcheck_dim,lerr_msg,ierr)
! Read in a string from vgosDB. May be an array.
! Exit with error if a problem.
      character*(*) lvar_name          !Name of string we want to read
      character, allocatable :: lstring(:)            !place where it is stored.
      integer ilen
      logical kcheck_dim               !check diminsion if  requiested.
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      integer j

! Local    
      integer num_dim                  !number of dimensions of vgosDB char array.
        
! Start of code 
      call get_netCDF_varid(lvar_name,ivarid,lerr_msg,ierr)
      if(ierr .ne. 0) return

      num_dim=num_dim_var(ivarid)
      
      if(num_dim .ne. 1) then     
          ierr=ivgosDB_number_dimensions_mismatch
          lerr_msg="2. Expecting scalar value for string '"//trim(lvar_name)//"' but got array"
          return
      endif 
   
      if(kcheck_dim) then
        if(ilen .ne. idim_size_var(ivar_dimid(num_dim,ivarid)) .and. &         
     &     ilen .ne. ivar_repeat(ivarid)) then           
          ierr=ivgosDB_dimension_size_mismatch
           write(lerr_msg,'("Expected dimension size ", i6, " but got ", i6, " for variable ",a)') &
         &    ilen, idim_size_var(ivar_dimid(num_dim,ivarid)),trim(lvar_name)
           return
        endif 
      else
        if(ivar_repeat(ivarid) .gt. 1) then
           ilen=ivar_repeat(ivarid) 
        else
           ilen=idim_size_var(ivar_dimid(num_dim,ivarid))
        endif 
      endif

      if(allocated(lstring)) then
       write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (lstring)
      endif
      allocate(lstring(ilen)) 
      
! read in the string.   
      iretval=nf_get_var_text(ncid,ivarid,lstring)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
      
      if(ivar_repeat(ivarid) .gt. 1) then
!        write(*,*) "...Repeating: ", lvar_name        
        lstring(2:ivar_repeat(ivarid))=lstring(1) 
       endif 

   
      return
      end subroutine 
! ****************************************************************************************
      subroutine get_netCDF_stringv_1d_alloc(lvar_name,lstring,nelem,istr_len,kcheck_dim,lerr_msg,ierr)
! Allocate space for a string and read it in. 
! All elements in string have same  number of characters which is istr_len
! Exit with error if a problem.
      character*(*) lvar_name          !Name of string we want to read
      integer*2 istr_len
      character*(istr_len), allocatable :: lstring(:)            !place where it is stored.
      integer nelem                    !number of elements
      logical kcheck_dim               !check diminsion if  requiested.
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error

! Local
      integer num_dim                  !number of dimensions of vgosDB char array.  
      integer istr_len_got              !length of string we got (from file)
      integer nelem_got                 !number of elements we got.
        
! Start of code 
!     write(*,*) "ENTRY:  ",lvar_name 
      call get_netCDF_varid(lvar_name,ivarid,lerr_msg,ierr)
      if(ierr .ne. 0) return

      num_dim=num_dim_var(ivarid)    
         
      if(num_dim .eq. 2) then      
          nelem_got=idim_size_var(ivar_dimid(2,ivarid))
       else
          nelem_got=1         !strictly speaking only true if num_dim=1. 
          if(ivar_repeat(ivarid) .ge. 1) nelem_got=ivar_repeat(ivarid)
       endif              
    
      if(kcheck_dim) then
        if(nelem .eq. 1 .and. num_dim .eq. 1) then
           continue
        else if(nelem .ne. 1) then
          if(num_dim .ne. 2 .and. ivar_repeat(ivarid) .eq. 0) then 
            ierr=ivgosDB_number_dimensions_mismatch
            lerr_msg="3. Expecting scalar value for string '"//trim(lvar_name)//"' but got array"
            write(*,*) "Var repeat: ",  ivar_repeat(ivarid)
            write(*,*) "Str_len: ", istr_len 
            return
          else           
            if(nelem .ne. nelem_got) then
               ierr=ivgosDB_dimension_size_mismatch
               write(lerr_msg,'("get_netCDF_stringv_1d_alloc: Expected dimension size ", &
         &       i5, " but got ", i5, " for variable ",a)') &
         &     nelem,nelem_got,trim(lvar_name)
               return
            endif 
          endif
         endif
      else
        nelem=nelem_got
      endif

      if(allocated(lstring)) then
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (lstring)
      endif

      istr_len_got=idim_size_var(ivar_dimid(1,ivarid))
      if(num_dim .eq. 1 .and. ivar_repeat(ivarid) .eq. 0) then 
         if(istr_len_got .gt. istr_len) then        
            ierr=ivgosDB_dimension_size_mismatch 
             write(lerr_msg,'("get_netCDF_stringv_1d_alloc: Expected str_len ", &
         &       i5, " but got ", i5, " for variable ",a)') &
         &     istr_len, istr_len_got,trim(lvar_name)            
             stop
          endif 
      endif 
 
      allocate(lstring(nelem))
      lstring=" " 
       
! read in the string.   
      iretval=nf_get_var_text(ncid,ivarid,lstring)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
      if(ivar_repeat(ivarid) .gt. 1) then
         lstring(2:nelem)=lstring(1)                  
!         write(*,*) "...Repeating: ", lvar_name  
!         write(*,*) lstring      
       endif 
        
    
      return
      end subroutine   
! ****************************************************************************************
      subroutine get_netCDF_double(lvar_name,rvar,idim_size,kcheck_dim,lerr_msg,ierr)
! Exit with error if a problem.
 ! 2019Jan17 Introduce idim_size_tmp(1) because get_varid_check_shape expects an array.
!           we copy idim_size to it before call, and copy from it to idim_size after
!   
      character*(*) lvar_name          !Name of string we want to read
      real*8 rvar(:)   !
      integer idim_size
      logical kcheck_dim
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error   
      integer num_dim/1/   
      integer idim_size_tmp(1)         !must do because get_varid_check_shape expects an array

      idim_size_tmp(1)=idim_size     
      call get_varid_check_shape(lvar_name,ivarid,idim_size_tmp,num_dim,kcheck_dim,lerr_msg,ierr)
      idim_size=idim_size_tmp(1)
      if(ierr .ne. 0) return
 
! read in the string.   
      iretval=nf_get_var_double(ncid,ivarid,rvar)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)
       if(ivar_repeat(ivarid) .gt. 1) then
 !          write(*,*) "...Repeating: ", lvar_name
           rvar(2:ivar_repeat(ivarid))=rvar(1)
       endif 


      return
      end subroutine 
! ****************************************************************************************
      subroutine get_netCDF_double_2d(lvar_name,rvar,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in 2D integer array.
! Allocate the space if necessary
! Exit with error if a problem.
 
      character*(*) lvar_name          !Name of string we want to read
      double precision, allocatable :: rvar(:,:)   !
      integer idim_size(2) 
      logical kcheck_dim
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      integer j
    
! local
      integer num_dim/2/  
      call get_varid_check_shape(lvar_name,ivarid,idim_size,num_dim,kcheck_dim,lerr_msg,ierr)
      if(ierr .ne. 0) return
      
! read it in. 
      iretval=nf_get_var_double(ncid,ivarid,rvar)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)

      if(ivar_repeat(ivarid) .gt. 1) then
 !          write(*,*) "...Repeating: ", lvar_name      
           do j=2,ivar_repeat(ivarid)
             rvar(1:idim_size(1),j)=rvar(1:idim_size(1),1)
           end do 
       endif 

      return
      end subroutine   
! ****************************************************************************************
      subroutine get_netCDF_double_alloc(lvar_name,rvar,idim_size,kcheck_dim,lerr_msg,ierr)
! Allocate the space if necessary
! Exit with error if a problem.
! 2012Feb16  JMGipson. If idim_size=0, indicating a scalar, set it to 1. 
! 2019Jan17 Introduce idim_size_tmp(1) because get_varid_check_shape expects an array.
!           we copy idim_size to it before call, and copy from it to idim_size after
! 
      character*(*) lvar_name          !Name of string we want to read
      real*8,allocatable :: rvar(:)   !
      integer idim_size
      logical kcheck_dim
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      integer num_dim/1/

      integer idim_size_tmp(1)
 
      idim_size_tmp(1)=idim_size     
      call get_varid_check_shape(lvar_name,ivarid,idim_size_tmp,num_dim,kcheck_dim,lerr_msg,ierr)
      idim_size=idim_size_tmp(1)
       
      if(ierr .ne. 0) return
!      write(*,*) "num_dim ", num_dim 
      if(num_dim .eq. 0) idim_size=1

      if(allocated(rvar)) then
        writE(*,*) "Oh no!" 
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (rvar)
        write(*,*) "In file: ", trim(lnetCDF_file) 
      endif
!      write(*,*) "Hi there"
!      write(*,*) idim_size, num_dim
!      write(*,*) ivar_repeat(ivarid)
      allocate(rvar(idim_size))
       
! read in the array   
      iretval=nf_get_var_double(ncid,ivarid,rvar)     
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)
       if(ivar_repeat(ivarid) .gt. 1) then
!           write(*,*) "...Repeating: ", lvar_name
           rvar(2:ivar_repeat(ivarid))=rvar(1)
       endif 


      return
      end subroutine  
! ****************************************************************************************
      subroutine get_netCDF_double_2d_alloc(lvar_name,rvar,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in 2D integer array.
! Allocate the space if necessary
! Exit with error if a problem.
 
      character*(*) lvar_name          !Name of string we want to read
      double precision, allocatable :: rvar(:,:)   !
      integer idim_size(2) 
      logical kcheck_dim
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      integer j
    
! local
      integer num_dim/2/  
      call get_varid_check_shape(lvar_name,ivarid,idim_size,num_dim,kcheck_dim,lerr_msg,ierr)
      if(ierr .ne. 0) return  
      if(allocated(rvar)) then
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        write(*,*) "From file: ", trim(lnetCDF_file)   
        deallocate (rvar)
      endif
      allocate(rvar(idim_size(1),idim_size(2)))
      
! read it in. 
      iretval=nf_get_var_double(ncid,ivarid,rvar)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)

      if(ivar_repeat(ivarid) .gt. 1) then
!           write(*,*) "...Repeating: ", lvar_name      
           do j=2,ivar_repeat(ivarid)
             rvar(1:idim_size(1),j)=rvar(1:idim_size(1),1)
           end do 
       endif 

      return
      end subroutine  

! ****************************************************************************************
      subroutine get_netCDF_double_3d_alloc(lvar_name,var,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in 2D integer array.
! Allocate the space if necessary
! Exit with error if a problem.
  
      character*(*) lvar_name          !Name of string we want to read
      double precision, allocatable :: var(:,:,:)   !
      integer idim_size(3) 
      logical kcheck_dim
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error  

      integer num_dim/3/
    
      call get_varid_check_shape(lvar_name,ivarid,idim_size,num_dim,kcheck_dim,lerr_msg,ierr)
      if(ierr .ne. 0) return
    
      if(allocated(var)) then
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (var)
      endif

      allocate(var(idim_size(1),idim_size(2),idim_size(3)))
      
! read in the string.   
      iretval=nf_get_var_double(ncid,ivarid,var)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)

      return
      end subroutine
! ****************************************************************************************
      subroutine get_netCDF_double_var(lvar_name,var,lerr_msg,ierr)
! Exit with error if a problem.
 
      character*(*) lvar_name          !Name of string we want to read
      real*8     var
     
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      
      call get_netCDF_varid(lvar_name,ivarid,lerr_msg,ierr)
      if(ierr .ne.0) return
    
      iretval=nf_get_var_double(ncid,ivarid,var) 
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval) 
   
      return
      end subroutine   
! ****************************************************************************************
      subroutine get_netCDF_int2_var(lvar_name,ivar,lerr_msg,ierr)
! Exit with error if a problem.
 
      character*(*) lvar_name          !Name of string we want to read
      integer*2     ivar(*)
     
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      
      call get_netCDF_varid(lvar_name,ivarid,lerr_msg,ierr)
      if(ierr .ne.0) return
    
      iretval=nf_get_var_int2(ncid,ivarid,ivar) 
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
      
      if(ivar_repeat(ivarid) .gt. 1) then
!         write(*,*) "...Repeating: ", lvar_name
         ivar(2:ivar_repeat(ivarid))=ivar(1)
       endif 

      return
      end subroutine   
! ****************************************************************************************
      subroutine get_netCDF_int4_var(lvar_name,ivar,lerr_msg,ierr)
! Allocate the space if necessary
! Exit with error if a problem.
 
      character*(*) lvar_name          !Name of string we want to read
      integer*4     ivar(*)
     
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
   
      call get_netCDF_varid(lvar_name,ivarid,lerr_msg,ierr)
      if(ierr .ne.0) return

      iretval=nf_get_var_int(ncid,ivarid,ivar) 
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
      
      if(ivar_repeat(ivarid) .gt. 1) then
 !          write(*,*) "...Repeating: ", lvar_name
           ivar(2:ivar_repeat(ivarid))=ivar(1)
       endif       

      return
      end subroutine   
!
!
! ****************************************************************************************
      subroutine get_netCDF_int1_alloc(lvar_name,ivar,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in a string from vgosDB. (NOT a string array)
! Allocate the space if necessary
! Exit with error if a problem.
! 2019Jan17 Introduce idim_size_tmp(1) because get_varid_check_shape expects an array.
!           we copy idim_size to it before call, and copy from it to idim_size after
! 
 
      character*(*) lvar_name          !Name of string we want to read
      integer*1,allocatable :: ivar(:)   !
      integer idim_size
      logical kcheck_dim 
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error

      integer num_dim/1/
      integer idim_size_tmp(1)         !must do because get_varid_check_shape expects an array

      idim_size_tmp(1)=idim_size     
      call get_varid_check_shape(lvar_name,ivarid,idim_size_tmp,num_dim,kcheck_dim,lerr_msg,ierr)
      idim_size=idim_size_tmp(1)

      if(ierr .ne. 0) return
      if(num_dim .eq.  0) idim_size=1     
    
     if(allocated(ivar)) then
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (ivar)
      endif
      allocate(ivar(idim_size))
      
! read in the string.   
      iretval=nf_get_var_int1(ncid,ivarid,ivar)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
 !     write(*,*) "       ...succesfully read "//trim(lvar_name)
       if(ivar_repeat(ivarid) .gt. 1) then
 !          write(*,*) "...Repeating: ", lvar_name
           ivar(2:ivar_repeat(ivarid))=ivar(1)
       endif 

      return
      end subroutine  

! ****************************************************************************************
      subroutine get_netCDF_int2_alloc(lvar_name,ivar,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in a string from vgosDB. (NOT a string array)
! Allocate the space if necessary
! Exit with error if a problem.
! 2019Jan17 Introduce idim_size_tmp(1) because get_varid_check_shape expects an array.
!           we copy idim_size to it before call, and copy from it to idim_size after
! 
 
      character*(*) lvar_name          !Name of string we want to read
      integer*2,allocatable :: ivar(:)   !
      integer idim_size
      logical kcheck_dim 
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      integer num_dim/1/
      integer idim_size_tmp(1)         !must do because get_varid_check_shape expects an array

      idim_size_tmp(1)=idim_size     
      call get_varid_check_shape(lvar_name,ivarid,idim_size_tmp,num_dim,kcheck_dim,lerr_msg,ierr)
      idim_size=idim_size_tmp(1)
     
      if(ierr .ne. 0) return
      if(num_dim .eq.  0) idim_size=1     
    
     if(allocated(ivar)) then
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (ivar)
      endif
      allocate(ivar(idim_size))
      
! read in the string.   
      iretval=nf_get_var_int2(ncid,ivarid,ivar)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!     write(*,*) "       ...succesfully read "//trim(lvar_name)
      if(ivar_repeat(ivarid) .gt. 1) then
!         write(*,*) "...Repeating: ", lvar_name
         ivar(2:ivar_repeat(ivarid))=ivar(1)
       endif 

      return
      end subroutine  
!****************************************************************************************
      subroutine get_netCDF_int2_2d(lvar_name,ivar,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in 2D integer array.
! Allocate the space if necessary
! Exit with error if a problem.
 
      character*(*) lvar_name          !Name of string we want to read
      integer*2  ivar(:,:) 
      integer idim_size(2) 
      logical kcheck_dim
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      integer j
    
! local
      integer num_dim/2/  
      call get_varid_check_shape(lvar_name,ivarid,idim_size,num_dim,kcheck_dim,lerr_msg,ierr)
      if(ierr .ne. 0) return
      
! read it in. 
      iretval=nf_get_var_int2(ncid,ivarid,ivar) 
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)

       if(ivar_repeat(ivarid) .gt. 1) then
!           write(*,*) "...Repeating: ", lvar_name      
           do j=2,ivar_repeat(ivarid)
             ivar(1:idim_size(1),j)=ivar(1:idim_size(1),1)
           end do 
       endif 


      return
      end subroutine   
! ****************************************************************************************
      subroutine get_netCDF_int2_2d_alloc(lvar_name,ivar,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in 2D integer array.
! Allocate the space if necessary
! Exit with error if a problem.
  
      character*(*) lvar_name          !Name of string we want to read
      integer*2,allocatable :: ivar(:,:)   !
      integer idim_size(2) 
      logical kcheck_dim
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error  
      integer j

      integer num_dim/2/
    
      call get_varid_check_shape(lvar_name,ivarid,idim_size,num_dim,kcheck_dim,lerr_msg,ierr)
      if(ierr .ne. 0) return
    
      if(allocated(ivar)) then
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (ivar)
      endif

      allocate(ivar(idim_size(1),idim_size(2)))
      
! read in the string.   
      iretval=nf_get_var_int2(ncid,ivarid,ivar)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)
       if(ivar_repeat(ivarid) .gt. 1) then
!           write(*,*) "...Repeating: ", lvar_name      
           do j=2,ivar_repeat(ivarid)
             ivar(1:idim_size(1),j)=ivar(1:idim_size(1),1)
           end do 
       endif 


      return
      end subroutine 
!
! ****************************************************************************************
      subroutine get_netCDF_int_alloc(lvar_name,ivar,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in a string from vgosDB. (NOT a string array)
! Allocate the space if necessary
! Exit with error if a problem.
! 2019Jan17 Introduce idim_size_tmp(1) because get_varid_check_shape expects an array.
!           we copy idim_size to it before call, and copy from it to idim_size after
! 
 
      character*(*) lvar_name          !Name of string we want to read
      integer,allocatable :: ivar(:)   !
      integer idim_size
      logical kcheck_dim
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error
      integer num_dim/1/
      integer idim_size_tmp(1)         !must do because get_varid_check_shape expects an array
 
      idim_size_tmp(1)=idim_size     
      call get_varid_check_shape(lvar_name,ivarid,idim_size_tmp,num_dim,kcheck_dim,lerr_msg,ierr)
      idim_size=idim_size_tmp(1)

    
     if(allocated(ivar)) then
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (ivar)
      endif
      allocate(ivar(idim_size))
      
! read in the string.   
      iretval=nf_get_var_int(ncid,ivarid,ivar)      
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)
       if(ivar_repeat(ivarid) .gt. 1) then
!           write(*,*) "...Repeating: ", lvar_name
           ivar(2:ivar_repeat(ivarid))=ivar(1)
       endif 

      return
      end subroutine  
! ****************************************************************************************
      subroutine get_netCDF_int_2d_alloc(lvar_name,ivar,idim_size,kcheck_dim,lerr_msg,ierr)
! Read in 2D integer array.
! Allocate the space if necessary
! Exit with error if a problem.
  
      character*(*) lvar_name          !Name of string we want to read
      integer,allocatable :: ivar(:,:)   !
      integer idim_size(2) 
      logical kcheck_dim 
      character*(*) lerr_msg 
      integer ierr                     !<>0 some error  
      integer j

      integer num_dim/2/
    
      call get_varid_check_shape(lvar_name,ivarid,idim_size,num_dim,kcheck_dim,lerr_msg,ierr)
      if(ierr .ne. 0) return
    
      if(allocated(ivar)) then
        write(*,*) "Warning: Deallocating space for "//trim(lvar_name)
        deallocate (ivar)
      endif

      allocate(ivar(idim_size(1),idim_size(2)))
      
! read in the string.   
      iretval=nf_get_var_int(ncid,ivarid,ivar)
      if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)
!      write(*,*) "       ...succesfully read "//trim(lvar_name)
   
       if(ivar_repeat(ivarid) .gt. 1) then
!          write(*,*) "...Repeating: ", lvar_name      
           do j=2,ivar_repeat(ivarid)
             ivar(1:idim_size(1),j)=ivar(1:idim_size(1),1)
           end do 
       endif 

      return
      end subroutine  

! ********************************************************************************************
     subroutine get_varid_check_shape(lvar_name,ivarid,idim_size,num_dim,kcheck_dim,lerr_msg,ierr)
! Get the variable id (ivarid) 
!  and check that the shapes agree. 
     character*(*) lvar_name          !Name of string we want to read
     integer*2,allocatable :: ivar(:,:)   !
     integer ivarid 
     integer idim_size(*)
     integer num_dim
     logical kcheck_dim             !if true,  check shape
     character*(*) lerr_msg 
     integer ierr                     !<>0 some error
! local
     integer i
     integer nch 
     integer iret 

      call get_netCDF_varid(lvar_name,ivarid,lerr_msg,ierr)
      if(ierr .ne. 0) return

      if(kcheck_dim) then    
        if(num_dim .eq.  1 .and.  idim_size(1) .eq. 1 .and.  num_dim_var(ivarid) .eq. 0) then
           continue
        else if(num_dim_var(ivarid) .eq. num_dim .or. &
     &          (num_dim_var(ivarid) .eq. num_dim-1 .and. ivar_repeat(ivarid) .gt. 1)) then          
          continue
        else
          ierr=ivgosDB_number_dimensions_mismatch
        
           write(lerr_msg,'("For ",a, " expected num_dim= ",i4," but got: ", i4)') &
     &            trim(lvar_name), num_dim, num_dim_var(ivarid)
          return
        endif
       
        do i=1,num_dim-1
          if(idim_size(i) .ne. idim_size_var(ivar_dimid(i,ivarid))) goto 500
        end do        
! If the last dimension doesn't match, check for "Constant" attribute
        if(idim_size(num_dim) .ne. idim_size_var(ivar_dimid(num_dim,ivarid))) then   
            if(idim_size(num_dim) .ne. ivar_repeat(ivarid)) goto 500                       
        endif       
      else
! Don't check the shape. just return it. 
        num_dim=num_dim_var(ivarid)
        do i=1,num_dim
          idim_size(i)=idim_size_var(ivar_dimid(i,ivarid))
        end do
        if(ivar_repeat(ivarid) .ne.0) then
          if(idim_size(num_dim) .ne. 1) num_dim=num_dim+1
          idim_size(num_dim)=ivar_repeat(ivarid)          
        endif 
      end if 
 !     if(ivar_repeat(ivarid) .gt. 1) write(*,*) "Must repeat: ", lvar_name 
      ierr=0   
      return

500   continue
      ierr=ivgosDB_dimension_size_mismatch

       write(lerr_msg,'("For variable ",a, " expected array size: ", 4i6)') &
     &         trim(lvar_name), idim_size(1:num_dim)
       nch=len_trim(lerr_msg)
       write(lerr_msg(nch+1:), '( "  Got:  ",4i6)') idim_size_var(ivar_dimid(1:num_dim,ivarid))

       return
       end subroutine 

! *******************************************************************************
      subroutine get_dr23(idr2_dimid,idr3_dimid)  !get these dimids if they exist. 
      integer idr2_dimid,idr3_dimid
! function
      integer iwhere_in_string_list

      idr2_dimid=iwhere_in_string_list(ldim_name_cap,num_dimid,"MAX_DERIV2")  !get the dimids for these if they exist.
       if(idr2_dimid .eq. 0) then
         idr2_dimid=iwhere_in_string_list(ldim_name_cap,num_dimid,"TIMEDIM2")  !get the dimids for these if they exist.
       endif 

       idr3_dimid=iwhere_in_string_list(ldim_name_cap,num_dimid,"MAX_DERIV3")
       if(idr3_dimid .eq. 0) then
         idr3_dimid=iwhere_in_string_list(ldim_name_cap,num_dimid,"TIMEDIM3")  !get the dimids for these if they exist.
       endif 
       return
       end subroutine 

!*********************************************************************8
      subroutine allocate_and_fill_int(ivalue4, isize, ifill)
      integer, allocatable :: ivalue4(:)
      integer isize
      integer ifill
      allocate(ivalue4(isize))
      ivalue4=ifill
      return
      end subroutine

!*********************************************************************8
      subroutine allocate_and_fill_real8(value8, isize, fill)
      real*8, allocatable :: value8(:)
      integer isize
      real*8 fill
      allocate(value8(isize))
      value8=fill
      return
      end subroutine 

! ***********************************
      end module
      
