      PROGRAM REPORT_LAST_WRAPPER_VERSION_AND_FILE
!
!     REPORT_LAST_WRAPPER_VERSION_QNE_FILE reports the specific version number
!     of a session's last version based on its last disk file
!     in the expected area.  In verbose mode, the file name is also reported.
!
!     run report_last_wrapper_version_and_file input_database_key file_identifier <verbosity_brevity> <communication_file>
!      where
!        input_database has the form YYMMMDDcc  AND
!        file_identifier is one of 
!           a or A to look at all wrapper files or
!           IVS to look at the wrapper file names that contain IVS or 
!           an IVS component id such as GSI or USN that is
!              contained in the wrapper file name(s) and restricts the
!              search to that subset of wrapper files.
!        verbosity_brevity is an optional argument with values of
!            V for verbose outout (default)
!            B for brief 
!        communication_file is the file which will receive the output
!          This is an optional argument.  If no file is specified, 
!          the output is written to the terminal.
!
!     written by kdb  171111
!
!     modifications
!
!     191105 kdb Add the ability to limit the search to a subset of 
!                wrapper files that contain an identifier 
!                (IVS or an analysis center or correlator id
!                such as GSI or USN).
!
!     RESTRICTIONS AND ASSUMPTIONS 
!
!      wrappers are assumed to be upper case
!   
      IMPLICIT NONE
!
      character*10 input_database_key
      integer i_len
      integer  idb_len
      integer ilu_out
      integer ict
      integer iunit
      character*1 output_format
      character*255 communication_file
      integer icom_file_len
      character*4 output_type, dir_structure
      integer*4 kerr
      integer ilast_ver
      character*255 message,output_message
      character*124 wrapper_file_root_dir
      character*100 last_wrapper_file_name
      integer*4 ierr4
      character database_key_nine_characters*9
      character*20 wrapper_file_id_request, wrapper_file_id
      integer wrapper_file_id_request_length
!
      ilu_out = 6
      if (iargc().eq.0) then
        write(ilu_out,"('run report_last_wrapper_version_and_file ', &
     &   '[session_key] [file_id] {v_or_b} {output_file}, where')")
        write(ilu_out,"('  session key is YYMONDYcc (no leading $)')")
        write(ilu_out,"('  file_id is a or A to look at all files or ')")
        write(ilu_out,"('    IVS to look at all wrapper files that contain IVS or ')")
        write(ilu_out,"('    an IVS component id such as GSF or USN ')")
        write(ilu_out,"('    that is contained in the wrapper file names ')")
        write(ilu_out,"('    and limits the search to those wrapper files ')")
        write(ilu_out,"('  v_or_b is v for verbose output (default) ', &
    &    'and b for brief')")
        write(ilu_out,"('  output_file is the full path to a file ', &
    &    'to receive output  ')")
        write(ilu_out,"('   (or blank to send output to the terminal)')")
        stop
      endif
!
      call rcpar( INT2(1), input_database_key )
      call casefold(input_database_key)
      idb_len = i_len(input_database_key)
      if (input_database_key(1:1) .eq. '$') then
        database_key_nine_characters = input_database_key(2:10)
      else
        database_key_nine_characters = input_database_key(1:9)
      endif
!
      if (iargc() .eq. 1) then
!       The default is to look at all wrapper files
        wrapper_file_id = ' '
      else
        call rcpar ( INT2(2), wrapper_file_id_request )
        wrapper_file_id_request_length = i_len(wrapper_file_id_request)
        wrapper_file_id = 'X'
        if (wrapper_file_id_request_length .eq. 1) then
          if (wrapper_file_id_request(1:1) .eq. 'a' .or. &
    &         wrapper_file_id_request(1:1) .eq. 'A') then
            wrapper_file_id = ' '
          endif
        else if (wrapper_file_id_request_length .gt. 2) then
          wrapper_file_id = wrapper_file_id_request
        endif
      endif
!
      if (wrapper_file_id .eq. 'X') then
        write(ilu_out,"('argument 2, wrapper_file_id_request is wrong ')") 
        write(ilu_out,"('  This must be either a or A for all wrappers  ',  &
     &    'or a wrapper id of 3+ characters ')") 
        write(ilu_out,"('  Quitting.')") 
        stop
      endif
! 
!     default is verbose output
      output_format = 'V'
      if (iargc().gt.2) then
        call rcpar( INT2(3), output_format )
        call casefold(output_format)
!       if requested, set up brief output
        if (output_format(1:1) .eq. 'B') output_format = 'B'
      endif
!
      if (iargc().gt.3) then
        call rcpar( INT2(4), communication_file )
        icom_file_len = i_len(communication_file)
        output_type = 'file'
      else
        output_type = 'term'
      endif
      wrapper_file_root_dir = ' '
      call getenvar ('VGOSDB_DIR', wrapper_file_root_dir)
      if (wrapper_file_root_dir .ne. ' ') then
        dir_structure = 'year'
        call find_last_wrapper_name_and_version &
     &    (database_key_nine_characters,wrapper_file_id,wrapper_file_root_dir, &
     &     dir_structure,last_wrapper_file_name,ilast_ver,message,kerr)
      else
        message = 'must set environment variable VGOSDB_DIR'
        kerr = -50
      endif
!
!     Set up the output message
!
      if (output_format .eq. 'B') THEN
!       brief output was requested.  Just report the version number 
!        (positive) or 0 for no versions or the negative error return.
        if (kerr .eq. 0) then
          write(output_message,"(i5)") ilast_ver
        else
          write(output_message,"(i5)") kerr
        endif
      else
!       verbose output was requested
        if (kerr .eq. 0) then
          write(output_message, &
     &      "(a9, ' last version is ',i5, ' file name is ',a)") &
     &         database_key_nine_characters, ilast_ver, &
     &     last_wrapper_file_name(1:i_len(last_wrapper_file_name))
        else
          write(output_message, &
     &     "(a9, ' error return is ', i5, ' reason: ',a)") &
     &         database_key_nine_characters, kerr, &
     &       message(1:i_len(message))
        endif
      endif
!
!     Report the output message.
!
      if (output_type .eq. 'file') then
        iunit = 20
        open(iunit,file=communication_file(1:icom_file_len), &
     &     iostat=ierr4,err=900, &
     &     status='unknown',access='sequential',form='formatted')
 900    if (ierr4 .ne.0) then
          write(ilu_out, "('report_last_version_wrapper_and_file ', &
     &      'cannot open the communication file due to error ', &
     &         i5)") ierr4
          write(ilu_out,"( &
     &       'so it will report the status of your request here: ')")
          write(ilu_out,"(a)") output_message(1:i_len(output_message))
        else
          write(iunit,"(a)") output_message(1:i_len(output_message))
          close(iunit)
        endif
      else
!       write to terminal
        write(ilu_out,"(a)") output_message(1:i_len(output_message))
      endif
!
      END
