      Program vgosDB_TOC
! Write out TOC for vgosDB session
      use vgosDB_variable
      implicit none 
      include 'netcdf.inc'

! History
!  2016Dec28 JMGipson  First version
!  2016Oct25 JMGipson  modified so it will write out results even if definition is missing
!  2019Jun06 JMGipson.  In get_netCDF_TOC was not initializing lu_out

      character*128 lerr_msg       !error message
      integer ierr                 ! ierr<>0  some error

      character*128 ldum                !input string
      integer, parameter :: max_token=7
      character*30 ltoken(max_token)
      integer num_token
      

      character*128 lwrapper    !wrapper file
      character*128 lvar_file    !File containing variables. 
      character*128 lfilout      !output file

      logical kexist
      integer, parameter :: max_list=600
      character*20 lname           !what are the names
      character*2  lband           !Values are "X","S"," "
      character*10 ltemp 
      character*12 lscope          !Session,Station,Scan,Observation
      type(vgosDB_var) vgosDB_list(max_list)   
      integer num_list       
      integer i      

      character*128 lversion/"vgosDB_TOC  v2018Oct25    JMGipson"/
   
      integer lu_in/10/

      write(*,*) trim(lversion)

      call parse_cmd_line(lwrapper,lfilout,lerr_msg,ierr)
      if(ierr .ne. 0) then
         write(*,*) trim(lerr_msg)
         stop
      endif

! Success 
      write(*,*) "Wrapper:        "//trim(lwrapper)  
      writE(*,*) "Output file:    "//trim(lfilout) 
      write(*,*)  " "  

! Extract the stuff from the wrapper
     call get_netCDF_TOC(lwrapper,lfilout,vgosDB_list,num_list,max_list,lerr_msg, ierr) 
     end

!
!********************************************************************************
      subroutine parse_cmd_line(lwrapper,lfilout,lerr_msg,ierr)
      use vgosDB_variable
      implicit none 
      character*(*) lwrapper
      character*(*) lfilout
      character*(*) lerr_msg       !error condition. 
      integer ierr                 !<>0 means some errror. All fatal. 
! local
      character*128 lcmd_file      !file containing lwrapper, lvar_file, lfilout
      logical kexist               !does a file exist
      integer lu_in
      integer i                    !counter 
      integer nch                  !# of characters in some string 
      integer ind                  !index of where one string is in another 
      character*128 ldum           !temporary character variable
      character*128 lwrapper_full  !full path to wrapper 
       
      ierr=0            !initialize to no error
      lerr_msg= " " 
      lfilout=" "   !initialize to 'not found' 

! Read the argument list and get the three files. 
! If no arguements print help screen
      select case(iargc())
      case(0) 
        write(*,*) "USAGE: vgosDB_TOC  Cmd_file"
        write(*,*) " -OR-  vgosDB_TOC Wrapper_file [Output_file]"
        write(*,*) " "
        write(*,*) "If output file is not specified, then is same is wrapper with .wrp-->.toc"
        write(*,*) " and is placed in the current directory." 
        write(*,*) " "    
        stop
      case(1)
! If a single argument, then could be control file or wrapper. 
        call getarg(1, lcmd_file)
        ind=index(lcmd_file,".wrp")
        if(ind .ne. 0) then
          lwrapper=lcmd_file    !its a wrapper. 
          goto 200      
        endif            
          
! Initialization        
        inquire(file=lcmd_file,exist=kexist)
        if(.not.kexist) then
           write(*,*) "Did not find control file: ",trim(lcmd_file)
           stop
         endif
! Open the command file and read. 
        open(lu_in,file=lcmd_file) 
        i=0
5       continue
        read(lu_in,'(a)',end=7) ldum
        if(ldum .eq. " ") goto 5
        if(ldum(1:1) .eq. "*" .or. ldum(1:1) .eq. "!") goto 5
        i=i+1
        select case(i)
        case(1)
          lwrapper=ldum   
        case(2)
          lfilout=ldum   
        case default
          lerr_msg="Too many non-comment lines in command file!"
          ierr=-1
          return 
        end select
        goto 5
7       continue
        close(lu_in)
      case(2) 
        call getarg(1, lwrapper)    
        call getarg(2, lfilout)   
      case default
        lerr_msg= "Bad number of arguments in command line."
        ierr=-2
        return 
      end select
! Handle the case if the output file is  " ".
! In this case make it look like the wrapper, but A) strip the directory. B) .wrp-->.dmp
!
200   continue 
      call make_wrapper_path(lwrapper,lwrapper_full)
      lwrapper=lwrapper_full
      if(lfilout .eq. " ") then
        nch=len_Trim(lwrapper)
        do i=nch,1,-1
          if(lwrapper(i:i) .eq. "/") then
              lfilout=lwrapper(i+1:)
              goto 9
           endif
        end do
        lerr_msg="Could not make name for lfilout from wrapper file: "//trim(lwrapper)
        ierr=-3
        return  
9       continue
        ind=index(lfilout,".wrp")
        if(ind .ne. 0) then
          lfilout(ind:ind+3) = ".toc"
!          write(*,*) "Output file: ",trim(lfilout)
        else
          lerr_msg="Did not find extesion '.wrp' on wrapper file: "//trim(lwrapper)
          ierr=-4
          return         
        endif 
      endif      

! See if the wrapepr exists     
      inquire(file=lwrapper,exist=kexist) 
      if(.not.kexist) then
         lerr_msg ="Wrapper not found: "//trim(lwrapper)
         ierr=-5
         return
      endif   

   
      return
      end subroutine 
! ****************************************************************************************
      subroutine get_netCDF_TOC(lwrapper,lfilout,vgosDB_list,num_list,max_list,lerr_msg,ierr)  
      use vgosDB_variable
      use netCDF_file_read
!      use nc_header
! now open the wrapper, read the files and return all of the variables.
! 
      implicit none 
! 2020Jun23 JMG. Iniatilze kprogram, khistory, ksolve 

      character*(*) lwrapper
      character*(*) lfilout
      type(vgosDB_var) vgosDB_list(max_list)
      integer num_list       
      integer max_list              !maximum number in list
      character*(*) lerr_msg        !Error message   
      integer ierr 

! Functions
      integer iwhere_in_string_list

! local
      logical kopen                  !is a section open
      logical kprogram               !is the program section open
      logical ksolve                 !Are we doing the solve program section.
      logical khistory               !is the history section open
      logical kfile                  !did we find a file.
      character*20 lprogram 
      character*64 lsection          !section name
      character*64 lkeyword          !
      character*8 lstation           !station name
      character*20 lsession          !session name
      character*20 lsessionAlt       !alternative name 
      logical kexist                 !Does a file exist
      character*128 ldefault_dir(0:3)  !Default directory. Note different defaults at different levels. 
      character*128 lfilnam          !name of  file.
      character*129 lfilnam_tot      !complete filename
      character*128 ldum             !holds a line of input
      integer nch_sec                !number of characters 
      character*40 lstub    
      character*40 lstub_tmp      
      character*20 lkind             !If there are several variants of a file, this contains the type. (NMF,CHAO,etc)
      character*2 lband              !band
      character*20 larray     
      integer nch                    !number of characters in some string (usually number of non-zero) 
      integer i                      !counter 
      character*10 lTimeTag          !kind of time tag.
      integer iwhere 

      logical khead_found            !indicate that the head was found.
      integer ind                    !index of substring in string
      integer ind_underscore, ind_dot   !locaation of underscore, or dot.
      integer ilen 
      integer istat_file
      integer ilevel               
      integer lu_wrp/13/
      logical kscope_open
      integer iline_num 
      character* 10 ltemp 
      integer lu_out/15/

! used to parse input line 
! Used to parse line read from file. 
      integer max_token,num_token
      parameter(max_token=10)
      character*128 ltoken(max_token)
      character*1 lchar
      integer isize1                  !size of first output dimension
      integer iatt 

! NetCDF variables that we ignore:
      integer, parameter :: num_ignore=14
      character*20 lignore_var(num_ignore) &
     & /"Stub","CreateTime","CreatedBy","Program","Subroutine", &
     &  "DataOrigin", "Session","vgosDB_Version", "Station","TimeTag",&
     &  "TimeTagFile", "CalcVer","Kind","Band"/


! Dimensions of arrays we read in.
      integer num_dim_in      !#dims of netcdf array we read  in
      integer idim1, idim2    !Storage for vgosDB arrays 
!
      integer j 

! local   
      character*128 ldir_pre 
      logical kverbose/.false./

! Open the variable file
      inquire(file=lwrapper,exist=kexist) 
      if(.not. kexist) then
         write(*,*) "File not found!"
         write(*,*) "--> ",trim(lwrapper)
         stop
      endif   
! Now we read in the file..
      open(lu_out, file=lfilout)  

! Make the 'pre directory' 
      ldir_pre=lwrapper
      nch=len_Trim(lwrapper)
      do i=nch,1,-1
        if(ldir_pre(i:i) .eq. "/") then
           ldir_pre(i+1:) = " "             !This is the directory the wrapper is in. 
           goto 5
        endif
      end do
      if(i .le. 1) then
         write(*,*) "some problem: "
      endif
5     continue     
!      write(*,*) "Directory: "//trim(ldir_pre) 

      istat_file=0
      ldefault_dir(0)=ldir_pre
!
      kprogram=.false.
      khistory=.false. 
      ksolve = .false.
      open(lu_wrp,file=lwrapper)
      kopen=   .false.
      kscope_open = .true. 
      ilevel =0
      lsection=" "
      lstation=" "  
      iline_num=0
      khead_found = .false.
      Num_list=0
      

! This is where we read in the file, a line at a time. 
100   continue
!      write(*,*) iline_num, ierr, trim(lerr_msg)
      read(lu_wrp,'(a)',end=200) ldum
!      write(*,*) "LDUM: ", trim(ldum) 
      iline_num=iline_num+1
      if(ldum .eq. " " .or. ldum(1:1) .eq. "!") goto 100
!      if(khead_found) then
!         kverbose=.false. 
!         writE(*,*) db_RW%NumObs, "---",ldum(1:30)
!      endif    
!      write(*,*) trim(ldum) 
! Only parse line upto begining of comment. 
      ind=index(ldum,"!")
      if(ind .ne. 0) then 
         nch=ind-1
       else
         nch=len_trim(ldum)
      endif

      kfile=.false. 
      call splitntokens(ldum(1:nch),ltoken,Max_Token,Num_Token)  
      lkeyword=ltoken(1)                  
        
      if(kverbose)  write(*,*) "---->"//trim(ldum)
!      write(*,*) " Key->"//trim(lkeyword)
      call capitalize(lkeyword)   
      if(lkeyword .eq. "UNAVAILABLE") goto 100
      KEYWORD: Select case(lkeyword)
 
      CASE("!")
       continue     ! This is a comment       
      CASE("VERSION")
       continue  
      Case("BEGIN")      
        if(kopen) then
          lerr_msg="Can't open new section while "//TRIM(lsection)//" is open!"
          ierr=1    
          goto 100      
        endif
        ilevel=ilevel+1
        ldefault_dir(ilevel)=ldefault_Dir(ilevel-1)
        kopen=.true.
        if(Num_Token .lt. 2) then
          lerr_msg="Did not specify section on BEGIN"
          ierr=2         
        endif
        lsection=ltoken(2)
        call capitalize(lsection)
! Second level CASE. 
        Select Case(lsection)
        CASE("HISTORY")
          khistory=.true. 
          kopen=.false. 
 
        CASE("STATION") 
           if(Num_token .lt. 3) then
             lerr_msg="Did not specify Station_Name' in 'BEGIN STATION'"
             ierr=3                       
           endif
           lstation=ltoken(3)
        CASE("SESSION","SCAN","OBS","OBSERVATION")
           continue
        CASE("PROCESS")
           continue 
        CASE("PROGRAM")  !pick up the solve dependent stuff...
           kprogram=.true. 
           kopen=.false. 
           if(Num_token .lt. 3) then
             lerr_msg="Did not specify program name in 'BEGIN PROGRAM'"
             ierr=3                       
           endif
           lprogram=ltoken(3)
           call capitalize(lprogram)  
           if(lprogram .eq. "SOLVE") then 
              ksolve=.true.
           else
              ksolve=.false.
           endif 
        CASE default
           lerr_msg="vgosDBOpn: Invalid section type: "//trim(ltoken(2))
        END SELECT
  
!        ldir=ldir_pre
        if(num_token .le. 3) then
          if(kverbose) write(*,'("In section: ",3(1x,a))') (Trim(ltoken(i)),i=2,Num_token)
        endif 
      Case("END")          
         ilevel=ilevel-1
         if(.not.kopen) then
! Special case
           if(khistory) then
              If(Num_Token .lt. 2) then
                 write(*,*) "Did not properly close HISTORY section"
              endif  
              khistory=.false.
              goto 100             
           else if(kprogram) then
              kprogram =.false.
              if(Num_token .lt. 3) then
                 write(*,*) "Did not properly close PROGRAM section"
              else
                 call capitalize(ltoken(3)) 
                 if(ltoken(3) .ne. lprogram) then
                   write(*,*) "Mismatch between beginning and end program!"
                   write(*,*) "Begin= ",trim(lprogram), " End=",trim(ltoken(3))
                 endif 
              endif 
              goto 100
           endif                         
! Normal case
           if(num_token .eq. 2) then
              lerr_msg="ERROR: END section "//TRIM(lsection)//" without corresponding BEGIN"
           else
              lerr_msg="ERROR: END section "//TRIM(lsection)//" for station "&
     &             //trim(ltoken(3))//" without corresponding BEGIN"
           endif 
           ierr=11             
         endif
         if(Num_Token .lt. 2) then
            lerr_msg= "ERROR: END with no Section_Name specified!"
            ierr=12           
         endif
         call capitalize(ltoken(2))
         if(ltoken(2) .ne. lsection) then 
           lerr_msg="Mismatch between begin section = "//TRIM(lsection)//" and end section ="//ltoken(2)
           ierr=13
         endif
         if(lsection .eq. "STATION") then
            call capitalize(ltoken(3))
            if(lstation .ne. ltoken(3)) then
              lerr_msg="Station section mismatch between begin station "&
     &               //TRIM(lstation)//" and end station "//TRIM(ltoken(1))
              ierr=14               
            endif         
         endif
         if(lsection .eq. "PROCESS") lsection="HISTORY"
         kopen=.false. 
      case("DEFAULT_DIR")       
         if(ltoken(2)(1:1) .eq. "/") then
            ldefault_dir(ilevel)=ltoken(2)
         else
            ldefault_dir(ilevel)=trim(ldefault_dir(ilevel-1))//ltoken(2)
         endif 
         nch=LEN_TRIM(ldefault_dir(ilevel))
         if(ldefault_dir(ilevel)(nch:nch) .ne. "/") ldefault_dir(ilevel)(nch+1:nch+1)="/"
          
      case("CREATED","CREATEDBY","HISTORY") 
         if(.not.khistory) then 
            lerr_msg="KEYWORD "//TRIM(lkeyword)//" found in outside of HISTORY section!"
            ierr=30
          endif
      case("PROGRAM")
         continue            
      case("SESSION","ALTSESSIONID")
        if(lsection .ne. "SESSION") then
           lerr_msg="KEYWORD "//TRIM(lkeyword)//" found outside of SESSION section!"
           ierr=31
        endif

        if(lkeyword .eq. "SESSION")then
          lsession=ltoken(2)   
          call capitalize(lsession)   
        else
          lsessionalt=ltoken(2)
          call capitalize(lsessionalt)
        endif 

      case default
        if(khistory) goto 100        
        if(kprogram .and. .not. ksolve) goto 100
! all of the remaining kinds of keywords specify files
        if(num_token .gt. 4) then          
          lerr_msg="Too many tokens! "//Trim(ldum)
          ierr=33
          goto 120
        endif

        if(index(ldum, ".nc") .eq. 0) then
          lerr_msg="Expecting filename and none found: "//Trim(ldum)
          ierr=32
          goto 120
        endif
          
        lTimeTag=ltoken(num_token)
        call capitalize(lTimeTag)
        if(lTimeTag .eq. "IMPLICIT" .or. lTimeTag .eq. "EXPLICIT") then
           Num_token = Num_token -1
        else
          lTimeTag=" "
        endif

        lfilnam=ltoken(Num_Token)
        if(index(lfilnam,".nc")  .eq. 0) then
          lerr_msg="Expected filename: "//trim(ldum)
          ierr=34
        endif
          
!        write(*,*) "File :", trim(lfilnam)
        if(num_token .ge. 3) then
           lstub=ltoken(1)
           larray=ltoken(2)
        else if(num_token .eq. 2) then
           lstub=ltoken(1)
           larray=ltoken(1)
        else
           nch=index(lfilnam,"_") 
           if(nch .eq. 0) nch=index(lfilnam,".") 
             if(nch .eq. 0) then 
               ierr=35
               lerr_msg="Can't extract kind from filename: "//trim(lfilnam)
               goto 120
             endif
             lstub=lfilnam(1:nch-1)
             larray=lstub
          endif 

!          writE(*,*) "ldir: ",trim(ldefault_dir(ilevel))
!          write(*,*) "File: ",trim(lfilnam)       
        
          lfilnam_tot=TRIM(ldefault_dir(ilevel))//lfilnam 
          inquire(file=lfilnam_tot,exist=kexist)
          if(.not.kexist) then 
             ierr=20
             lerr_msg="File not found: "//TRIM(lfilnam_tot)
             goto 120
          else       
            if(kverbose) write(*,'(a)') "    ...found file "//TRIM(lfilnam_tot)//" Kind= " &
     &                //Trim(lstub)//" Array= "//trim(larray)
          endif 
 
          call capitalize(lstub)                           
          if(lstub .eq. "HEAD") then
             khead_found=.true.                             
          else                         
! Some other kind of file.  This is an error if we haven't opened the head yet.
            if(.not.khead_found) then
               ierr=60
               lerr_msg="In session "//trim(lsession)//" trying to open file before reading head." 
               return 
            endif
          endif 
! Extract the 'kind' if there is one
         call get_vgosDB_filename_arg(lfilnam,"k",lkind,ierr)
         call get_vgosDB_filename_arg(lfilnam,"b",lband,ierr)

! This is the file that we oen. 
         lnetCDF_file=lfilnam_tot
!         write(*,*) trim(lfilnam_tot)
         kverbose=.false.
         call read_netCDF_var_info(kverbose,lerr_msg,ierr)
!         write(*,*) "enter i"
!         read(*,*) i 

        write(lu_out,*) trim(lfilnam_tot) 
! Here is where we read the NETCDF and extract all of the varibales.
         do ivarid=1,num_varid
           iwhere=iwhere_in_string_list(lignore_var,num_ignore , lvar_name(ivarid))          
           if(iwhere .eq.  0) then 
             Num_list=Num_list+1  
             if(Num_list .gt. Max_list) then
                write(*,*) "Not enough space in vgosDB_list! Change max_list and recompile." 
                stop
             endif 
             vgosDB_list(Num_list)%lname=lvar_name(ivarid) 
             vgosDB_list(num_list)%ldefinition=" " 
             iretval=nf_get_att_text(ncid,ivarid,"Definition",vgosDB_list(num_list)%ldefinition) 
             if(iretval .ne. 0) then                 
               write(*,*) "Variable ", trim(lvar_name(ivarid)), " is missing definition! "
               vgosDB_list(num_list)%ldefinition="---No defintion given in vgosDB--"
!               stop 
             endif    

             vgosDB_list(num_list)%lcode="  " 
             iretval=nf_get_att_text(ncid,ivarid,"LCODE",vgosDB_list(num_list)%lCode)  
             ind=index(vgosDB_list(num_list)%lcode,char(0))
             if(ind .ne. 0) vgosDB_list(num_list)%lcode(ind:)=" " 
             write(lu_out,'(i4,1x, a,1x,a,1x,a)') Num_list, vgosDB_list(Num_list)%lname, vgosDB_list(Num_list)%lcode, & 
     &            vgosDB_list(Num_list)%ldefinition     

           endif 
         end do
         write(lu_out,*) " " 
 

! close netcdf file that is opened with read_netCDF_var
         iretval=nf_close(ncid)
         if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)       
        END SELECT KEYWORD
        goto 100 
        
120     continue
        if(ierr .ne. 0) then     
!           write(*,*) "Ierr: ", ierr 
!           write(*,*) "line 501: ",trim(ldum)     
           if(kverbose) write(*,*) "Exit Read_vgosDB error: ", ierr 
           close(lu_wrp)
           close(lu_out)
           return
        endif

200     continue       
        close(lu_out)
        close(lu_wrp)
        if(kverbose) write(*,*) "Exiting read_vgosDB without error!"
        return
        end    


