      Program vgosDB_dump_obs
! Program to read in a vgosDB file and then dump out some observation info.
      use vgosDB_variable
      implicit none 
      include 'netcdf.inc'
! 2019Jun06 JMGipson.  In parse_cmd_line set lu_in 
! 2020Jun24 JMG. Output header row to screen, some cleanup

      character*128 lerr_msg       !error message
      integer ierr                 ! ierr<>0  some error

      character*128 ldum                !input string
      integer, parameter :: max_token=7
      character*30 ltoken(max_token)
      integer num_token
      

      character*128 lwrapper    !wrapper file
      character*128 lvar_file    !File containing variables. 
      character*128 lfilout      !output file

      logical kexist
      integer, parameter :: max_list=50
      character*20 lname           !what are the names
      character*2  lband           !Values are "X","S"," "
      character*10 ltemp 
      character*12 lscope          !Session,Station,Scan,Observation
      type(vgosDB_var) vgosDB_list(max_list)   
      integer num_list       
      integer i      

      character*128 lversion/"vgosDB_dump  v2020Jun24    JMGipson"/
   
      integer lu_in/10/

      write(*,*) trim(lversion)

      call parse_cmd_line(lwrapper,lvar_file,lfilout,lerr_msg,ierr)
      if(ierr .ne. 0) then
         write(*,*) trim(lerr_msg)
         stop
      endif

! Success 
      write(*,*) "Wrapper:        "//trim(lwrapper)
      writE(*,*) "Var file:       "//trim(lvar_file)
      writE(*,*) "Output file:    "//trim(lfilout) 
      write(*,*)  " "  

!*******************************************************************************************************
      open(lu_in,file=lvar_file)
! Read in variable list
      num_list=0
      do while(.true.)
10      continue
        read(lu_in,'(a)', end=20,err=20) ldum
        if(ldum(1:1) .eq. "!") goto 10
        if(ldum .eq. " ") goto 10
        call capitalize(ldum) 
        call splitNTokens(ldum,ltoken,Max_Token,Num_Token)
        if(Num_Token .lt. 2) then
           write(*,*) "Must have at least two tokens (name, scope) in variable line: "
           write(*,*) "    "//trim(ldum)
           stop         
        endif 
        num_list=num_list+1 
        if(num_list .gt. max_list) then
          write(*,*) "Maximum number of variables is: ",max_list
          write(*,*) "Stopping with the number read in."
          num_list=num_list-1
          goto 20
        endif 

        vgosDB_list(num_list)%kfound=.false.            !set not found
        vgosDB_list(num_list)%lname=ltoken(1)            !name
        call capitalize(ltoken(2))
        vgosDB_list(num_list)%lscope=ltoken(2)           !scope
        vgosDB_list(num_list)%lband=" "                  !initialze to none. 

        if(Num_Token .ge.3) then 
            vgosDB_list(num_list)%lformat=ltoken(3)
         else
            vgosDB_list(num_list)%lformat="*"            !general format
         endif    

! Find the station or the band as appropriate
        if(vgosDB_list(num_list)%lscope .eq."STATION") then
           if(num_token .lt. 4) then
              write(*,*) "Need to specify the station name if the scope is STATION"
              stop
           else
              vgosDB_list(num_list)%lstation=ltoken(4)
           endif       
        else if(vgosDB_list(num_list)%lscope .eq. "OBSERVATION") then      
          vgosDB_list(num_list)%lband=" "   
          if(Num_Token .eq. 4) then 
            select case(ltoken(4))
            case("S","X","K","Ka","Ku")
               vgosDB_list(num_list)%lband=ltoken(4)
            case("-"," ")
               vgosDB_list(num_list)%lband=" "
            case default
               write(*,*) "Unknown band ",ltoken(4)
               stop 
            end select                      
           endif
        endif 
 
      end do 
20    continue
      close(lu_in)  

! Extract the stuff from the wrapper
     call read_netCDF_variables(lwrapper,vgosDB_list,num_list,lerr_msg, ierr) 

     do i=1,num_list   
       if(.not.vgosDB_list(i)%kfound) then 
         select case(vgosDB_list(i)%lscope)
         case("STATION")
            ltemp=vgosDB_list(i)%lstation         
         case("OBSERVATION") 
            if(vgosDB_list(i)%lband .ne. " ") ltemp=trim(vgosDB_list(i)%lband)//"-band"
         case default
            ltemp=" "
         end select
         write(*,'("Did not find: ", a, " ",a)') vgosDB_list(i)%lname,ltemp                   
       endif
      end do

      call write_vgosDB_dump(lfilout,lversion, lwrapper,vgosDB_list,num_list,lerr_msg,ierr)
      if(ierr .ne. 0) then
         write(*,*) lerr_msg
      endif
      stop
      end

!**************************************************************************************************
      subroutine write_vgosDB_dump(lfilout,lversion,lwrapper,vgosDB_list,num_list,lerr_msg,ierr)
      use vgosDB_variable
      implicit none 
      include 'netcdf.inc'

! passed
      character*(*) lfilout                    !file to write to
      character*(*) lversion                   !version info of calling program. 
      character*(*) lwrapper                   !where we got the information from 
      type(vgosDB_var) vgosDB_list(*)
      integer num_list
! returned 
      character*(*) lerr_msg
      integer ierr
! local
      integer lu_out/11/                       !lu for output file 
   
      integer i,j

      integer ivar_index(num_list)         !list of variables that we are writing to....
      integer ivar                        !index of variable that we are writing
      integer num_var 

      logical kstat_do(num_list)        !Do we need to do this station?
      character*8 lstation
 
      integer iscope                      !1=session, 2=station, 3=scan, 4=observation
      character*12 lscope(4)/"SESSION","STATION","SCAN","OBSERVATION"/
 
      open(lu_out,file=lfilout)
      write(lu_out,'(a)') trim(lversion)
      write(lu_out,'("Data taken from: ",a)') trim(lwrapper)
      
! initilize to no error
      ierr=0 

      do iscope=1,4
! Make list of variables to write out depending on scope.
        select case(iscope)
        case(1,3,4)
          num_var=0 
          do i=1,num_list      
            if(vgosDB_list(i)%lscope .eq. lscope(iscope) .and. vgosDB_list(i)%kfound) then
               num_var=num_var+1
              ivar_index(num_var)=i
            endif
          end do 
          if(num_var .gt. 0) then
            if(i .eq. 1) then
              call write_session_var_list(ivar_index,num_var, vgosDB_list,lu_out,lerr_msg,ierr)
            else
              call write_var_list(ivar_index,num_var, vgosDB_list,lu_out,lerr_msg,ierr)
            endif 
          endif 
! This is station dependent case. A little more complicated.
        case(2)
          kstat_do=.true.     !initilize to have to do all stations.
          do i=1,num_list                              
            if(vgosDB_list(i)%lscope .eq. lscope(iscope) .and. vgosDB_list(i)%kfound) then 
              lstation=vgosDB_list(i)%lstation
              num_var=0
              do j=i,num_list
                if(vgosDB_list(j)%lstation .eq. lstation .and. vgosDB_list(i)%kfound .and. kstat_do(j)) then 
                  num_var=num_var+1
                  ivar_index(num_var)=j
                  kstat_do(j) = .false.                 !set flag indicating that we are done so we don't repeat. 
                endif 
              end do
              if(num_var .gt. 0) then 
                 call write_var_list(ivar_index,num_var, vgosDB_list,lu_out, lerr_msg,ierr)
              endif
            endif
          end do
        end select 
      end do 
      close(lu_out) 
      end subroutine 

!*************************************************************************************
      subroutine write_var_list(ivar_index,num_var, vgosDB_list,lu_out,lerr_msg,ierr)
      use vgosDB_variable
      implicit none 
      include 'netcdf.inc'

      integer ivar_index(*)         !list of variables that we are writing to....
      integer num_var
      type(vgosDB_var) vgosDB_list(*)
      integer lu_out 
      character lerr_msg
      integer ierr
! local   
      integer ivar 
      integer ishape(2)
      integer num_out 
!      integer nch  
!      integer ind 
 
      integer i,j                         !counters
      character*64 lformat
      character*2 lband                   !band (X or S or Ku or ...)
      integer ilen_out(num_var)           !how much space to allocate for each variable...
      character*80 ltemp_out              !Temporary string that we write to.
      character*20 lscope
      character*1 lquote/'"'/

! Start of code 
      ivar=ivar_index(1)
       
! Find the dimensions
      select case(vgosDB_list(ivar)%itype)
      case(nf_double)             
        ishape=shape(vgosDB_list(ivar)%value8)        
      case(nf_byte)
        ishape=shape(vgosDB_list(ivar)%ivalue1)  
      case(nf_short)
        ishape=shape(vgosDB_list(ivar)%ivalue2) 
      case(nf_int)
        ishape=shape(vgosDB_list(ivar)%ivalue4) 
      case(nf_char)
        ishape=shape(vgosDB_list(ivar)%string)
      end select
     
      num_out=ishape(2)
      select  case(vgosDB_list(ivar)%lscope)
      case("STATION")
        write(lu_out,'("BEGIN STATION ",a," # SCANS=",i6)') trim(vgosDB_list(ivar)%lstation),num_out
      case("SCAN","OBSERVATION")
        write(lu_out,'("BEGIN ",a," # =",i6)') vgosDB_list(ivar)%lscope, num_out   
      end select 
     
! NOTE: Variable  names do not line up with scans. 
! We want to write stuff out so things line up nicely. However we do not know before the fact how much space we need.
! We figure this out by A) writing out the variable names and seeing how long they are.  B) write out data and see how much space we need.

      write(lu_out,'(" #  ",$)')
      do j=1,num_var
       ivar=ivar_index(j)
       lband=vgosDB_list(ivar)%lband
       if(lband .eq. "-") lband=" "

       if(vgosDB_list(ivar)%kfound) then 
         if(lband .eq. " ") then 
            write(lu_out,'(a," ",$)') trim(vgosDB_list(ivar)%lname)
         else
            write(lu_out,'(a," ",$)') trim(vgosDB_list(ivar)%lname)//"-"//trim(lband)
         endif 
         
       else  
         write(*,'("Did not find: ", a, " ",a)') vgosDB_list(ivar)%lname,lband                 
       endif
      end do
      write(lu_out,*) 

!     do i=1,10
      do i=1,num_out
        write(lu_out,'(i6," ",$)') i 
        do j=1,num_var
!          writE(*,*) vgosDB_list(ivar)%lname 
          ivar=ivar_index(j)
          if(vgosDB_list(ivar)%kfound) then 
            lformat="("//trim(vgosDB_list(ivar)%lformat)//",$)"
!            if(vgosDB_list(ivar)%itype .eq. nf_char) write(lu_out,"(a,$)") lquote      
        
            select case(vgosDB_list(ivar)%itype)
            case(nf_double)        
              write(lu_out,lformat,err=200) vgosDB_list(ivar)%value8(:,i)
            case(nf_byte)
              write(lu_out,lformat,err=200) vgosDB_list(ivar)%ivalue1(:,i)
            case(nf_short)
              write(lu_out,lformat,err=200) vgosDB_list(ivar)%ivalue2(:,i)
            case(nf_int)
              write(lu_out,lformat,err=200) vgosDB_list(ivar)%ivalue4(:,i)
            case(nf_char)
              write(lu_out,lformat,err=200) vgosDB_list(ivar)%string(:,i)
            end select
        
!           if(vgosDB_list(ivar)%itype .eq. nf_char) write(lu_out,"(a,$)") lquote 
            write(lu_out,'(" ",$)')  
          endif
        end do
        write(lu_out,*) " " 
      end do

      select case (vgosDB_list(ivar)%lscope)
      case("STATION")
        write(lu_out,'("END STATION ",a)') trim(vgosDB_list(ivar)%lstation)
      case("SCAN")
        write(lu_out,'("END SCAN")') 
      case("OBSERVATION")
        write(lu_out,'("END OBSERVATION")') 
      end select
      return 

! Come here on error writing out.
200   continue
      write(*,'("Can not write variable ",a, " with format=>> ",a," << and values=",$)') &
     &   trim(vgosDB_list(ivar)%lname),trim(vgosDB_list(ivar)%lformat) 
 
        select case(vgosDB_list(ivar)%itype)
        case(nf_double)        
          write(*,*) vgosDB_list(ivar)%value8(:,i)
        case(nf_byte)
          write(*,*) vgosDB_list(ivar)%ivalue1(:,i)
        case(nf_short)
          write(*,*) vgosDB_list(ivar)%ivalue2(:,i)
        case(nf_int)
          write(*,*) vgosDB_list(ivar)%ivalue4(:,i)
        case(nf_char)
          write(*,*) vgosDB_list(ivar)%string(:,i)
        end select
    
     stop 
      

      return       
      end subroutine 
!***********************************************************************************************
      subroutine write_session_var_list(ivar_index,num_var, vgosDB_list,lu_out,lerr_msg,ierr)
      use vgosDB_variable
      implicit none 
      include 'netcdf.inc'

      integer ivar_index(*)         !list of variables that we are writing to....
      integer num_var
      type(vgosDB_var) vgosDB_list(*)
      integer lu_out 
      character lerr_msg
      integer ierr
      end subroutine 

!********************************************************************************
      subroutine parse_cmd_line(lwrapper,lvar_file,lfilout,lerr_msg,ierr)
      use vgosDB_variable
      implicit none 
      character*(*) lwrapper
      character*(*) lvar_file
      character*(*) lfilout
      character*(*) lerr_msg       !error condition. 
      integer ierr                 !<>0 means some errror. All fatal. 
! local
      character*128 lcmd_file      !file containing lwrapper, lvar_file, lfilout
      logical kexist               !does a file exist
      integer lu_in/15/
      integer i                    !counter 
      integer nch                  !# of characters in some string 
      integer ind                  !index of where one string is in another 
      character*128 ldum           !temporary character variable
      character*128 lwrapper_full  !full path to wrapper
       
      ierr=0            !initialize to no error
      lerr_msg= " " 
      lfilout=" "   !initialize to 'not found'

! Read the argument list and get the three files. 
! If no arguements print help screen
      select case(iargc())
      case(0) 
        write(*,*) "USAGE: vgosDB_dump  Cmd_file"
        write(*,*) " -OR-  vgosDB_dump Wrapper_file Variable_file Output_file"
        write(*,*) " "
        write(*,*) "If output file is not specified, then is same is wrapper with .wrp-->.dmp"
        write(*,*) " and is placed in the current directory." 
        write(*,*) " " 
        write(*,*) "NOTE: Currently does not work with SESSION variables." 
        stop
      case(1)
! If a single argument, then it is the command file. 
        call getarg(1, lcmd_file)
! Initialization        
        inquire(file=lcmd_file,exist=kexist)
        if(.not.kexist) then
           write(*,*) "Did not find control file: ",trim(lcmd_file)
           stop
         endif
! Open the command file and read. 
        open(lu_in,file=lcmd_file) 
        i=0
5       continue
        read(lu_in,'(a)',end=7) ldum
        if(ldum .eq. " ") goto 5
        if(ldum(1:1) .eq. "*" .or. ldum(1:1) .eq. "!") goto 5
        i=i+1
        select case(i)
        case(1)
          lwrapper=ldum
        case(2)
          lvar_file=ldum
        case(3)
          lfilout=ldum
        case default
          lerr_msg="Too many non-comment lines in command file!"
          ierr=-1
          return 
        end select
        goto 5
7       continue
        close(lu_in)
      case(2) 
        call getarg(1, lwrapper)
        call getarg(2, lvar_file)
      case(3)
        call getarg(1, lwrapper)
        call getarg(2, lvar_file)
        call getarg(3, lfilout)
      case default
        lerr_msg= "Bad number of arguments in command line."
        ierr=-2
        return 
      end select

! Make the full wrapper name.  
      call make_wrapper_path(lwrapper, lwrapper_full) 
      lwrapper=lwrapper_full

! Handle the case if the output file is  " ".
! In this case make it look like the wrapper, but A) strip the directory. B) .wrp-->.dmp
!
      if(lfilout .eq. " ") then
        nch=len_Trim(lwrapper)
        do i=nch,1,-1
          if(lwrapper(i:i) .eq. "/") then
              lfilout=lwrapper(i+1:)
              goto 9
           endif
        end do
        lerr_msg="Could not make name for lfilout from wrapper file: "//trim(lwrapper)
        ierr=-3
        return  
9       continue
        ind=index(lfilout,".wrp")
        if(ind .ne. 0) then
          lfilout(ind:ind+3) = ".dmp"
!          write(*,*) "Output file: ",trim(lfilout)
        else
          lerr_msg="Did not find extension '.wrp' on wrapper file: "//trim(lwrapper)
          ierr=-4
          return         
        endif 
      endif      

! See if the wrapepr exists     
      inquire(file=lwrapper,exist=kexist) 
      if(.not.kexist) then
         lerr_msg ="Wrapper not found: "//trim(lwrapper)
         ierr=-5
         return
      endif   

! Check if the variable file exists 
      inquire(file=lvar_file,exist=kexist) 
      if(.not. kexist) then
         lerr_msg= "Variable file not found: "//trim(lvar_file)
         ierr=-6
         return 
      endif   
   
      return
      end subroutine 


! ****************************************************************************************
      subroutine read_netCDF_variables(lwrapper,vgosDB_list,num_list,lerr_msg,ierr)  
      use vgosDB_variable  
      use netCDF_file_read
!      use nc_header
! now open the wrapper and get  the the stuff
! Adapted from read_vgosDB.f in vgosDB library.
      implicit none 

! 2020Jun23 JMG. Iniatilze kprogram, khistory, ksolve 

      character*(*) lwrapper
      type(vgosDB_var) vgosDB_list(num_list)   
      integer num_list  
      character*(*) lerr_msg        !Error message   
      integer ierr 

! local
      logical kopen                  !is a section open
      logical kprogram               !is the program section open
      logical ksolve                 !Are we doing the solve program section.
      logical khistory               !is the history section open
      logical kfile                  !did we find a file.
      character*20 lprogram 
      character*64 lsection          !section name
      character*64 lkeyword          !
      character*8 lstation           !station name
      character*20 lsession          !session name
      character*20 lsessionAlt       !alternative name 
      logical kexist                 !Does a file exist
      character*128 ldefault_dir(0:3)  !Default directory. Note different defaults at different levels. 
      character*128 lfilnam          !name of  file.
      character*129 lfilnam_tot      !complete filename
      character*128 ldum             !holds a line of input
      integer nch_sec                !number of characters 
      character*40 lstub    
      character*40 lstub_tmp      
      character*20 lkind             !If there are several variants of a file, this contains the type. (NMF,CHAO,etc)
      character*2 lband              !band
      character*20 larray     
      integer nch                    !number of characters in some string (usually number of non-zero) 
      integer i                      !counter 
      character*10 lTimeTag          !kind of time tag.
      integer iwhere 

      logical khead_found            !indicate that the head was found.
      logical kheader_line           !output headerline?
      integer ind                    !index of substring in string
      integer ind_underscore, ind_dot   !location of underscore, or dot.
      integer ilen 
      integer istat_file
      integer ilevel               
      integer lu_wrp/13/
      logical kscope_open
      integer iline_num 
      character* 10 ltemp 

! used to parse input line 
! Used to parse line read from file. 
      integer max_token,num_token
      parameter(max_token=10)
      character*128 ltoken(max_token)
      character*1 lchar
      integer isize1                  !size of first output dimension

! Dimensions of arrays we read in.
      integer num_dim_in      !#dims of netcdf array we read  in
      integer idim1, idim2    !Storage for vgosDB arrays
  
!
      integer j 

! local   
      character*128 ldir_pre 
      logical kverbose/.false./

!      kverbose=.true.
! Open the variable file
      inquire(file=lwrapper,exist=kexist) 
      if(.not. kexist) then
         write(*,*) "File not found!"
         write(*,*) "--> ",trim(lwrapper)
         stop
      endif   
! Now we read in the file...  

! Make the 'pre directory' 
      ldir_pre=lwrapper
      nch=len_Trim(lwrapper)
      do i=nch,1,-1
        if(ldir_pre(i:i) .eq. "/") then
           ldir_pre(i+1:) = " "             !This is the directory the wrapper is in. 
           goto 5
        endif
      end do
      if(i .le. 1) then
         write(*,*) "some problem: "
      endif
5     continue     
!      write(*,*) "Directory: "//trim(ldir_pre) 

      istat_file=0
      ldefault_dir(0)=ldir_pre

     kprogram=.false.
     khistory=.false. 
     ksolve = .false.
     kheader_line=.false. 
!
     open(lu_wrp,file=lwrapper)
        kopen=   .false.
      kscope_open = .true. 
      ilevel =0
      lsection=" "
      lstation=" "  
      iline_num=0
      khead_found = .false.

! This is where we read in the file, a line at a time. 
100   continue
!      write(*,*) iline_num, ierr, trim(lerr_msg)
      read(lu_wrp,'(a)',end=200) ldum
!      write(*,*) "LDUM: ", trim(ldum) 
      iline_num=iline_num+1
      if(ldum .eq. " " .or. ldum(1:1) .eq. "!") goto 100
!      if(khead_found) then
!         kverbose=.false. 
!         writE(*,*) db_RW%NumObs, "---",ldum(1:30)
!      endif    
!      write(*,*) trim(ldum) 
! Only parse line upto begining of comment. 
      ind=index(ldum,"!")
      if(ind .ne. 0) then 
         nch=ind-1
       else
         nch=len_trim(ldum)
      endif

      kfile=.false. 
      call splitntokens(ldum(1:nch),ltoken,Max_Token,Num_Token)  
      lkeyword=ltoken(1)                  
        
      if(kverbose)  write(*,*) "---->"//trim(ldum)
!      write(*,*) " Key->"//trim(lkeyword)
      call capitalize(lkeyword)   
      if(lkeyword .eq. "UNAVAILABLE") goto 100
      KEYWORD: Select case(lkeyword)
 
      CASE("!")
       continue     ! This is a comment       
      CASE("VERSION")
       continue  
      Case("BEGIN")      
        if(kopen) then
          lerr_msg="Can't open new section while "//TRIM(lsection)//" is open!"
          ierr=1    
          goto 100      
        endif
        ilevel=ilevel+1
        ldefault_dir(ilevel)=ldefault_Dir(ilevel-1)
        kopen=.true.
        if(Num_Token .lt. 2) then
          lerr_msg="Did not specify section on BEGIN"
          ierr=2         
        endif
        lsection=ltoken(2)
        call capitalize(lsection)
! Second level CASE. 
        Select Case(lsection)
        CASE("HISTORY")
          khistory=.true. 
          kopen=.false. 
 
        CASE("STATION") 
           if(Num_token .lt. 3) then
             lerr_msg="Did not specify Station_Name' in 'BEGIN STATION'"
             ierr=3                       
           endif
           lstation=ltoken(3)
        CASE("SESSION","SCAN","OBS","OBSERVATION")
           continue
        CASE("PROCESS")
           continue 
        CASE("PROGRAM")  !pick up the solve dependent stuff...
           kprogram=.true. 
           kopen=.false. 
           if(Num_token .lt. 3) then
             lerr_msg="Did not specify program name in 'BEGIN PROGRAM'"
             ierr=3                       
           endif
           lprogram=ltoken(3)
           call capitalize(lprogram)  
           if(lprogram .eq. "SOLVE") then 
              ksolve=.true.
           else
              ksolve=.false.
           endif 
        CASE default
           lerr_msg="vgosDBOpn: Invalid section type: "//trim(ltoken(2))
        END SELECT
  
!        ldir=ldir_pre
        if(num_token .le. 3) then
          if(kverbose) write(*,'("In section: ",3(1x,a))') (Trim(ltoken(i)),i=2,Num_token)
        endif 
      Case("END")          
         ilevel=ilevel-1
         if(.not.kopen) then
! Special case
           if(khistory) then
              If(Num_Token .lt. 2) then
                 write(*,*) "Did not properly close HISTORY section"
              endif  
              khistory=.false.
              goto 100             
           else if(kprogram) then
              kprogram =.false.
              if(Num_token .lt. 3) then
                 write(*,*) "Did not properly close PROGRAM section"
              else
                 call capitalize(ltoken(3)) 
                 if(ltoken(3) .ne. lprogram) then
                   write(*,*) "Mismatch between beginning and end program!"
                   write(*,*) "Begin= ",trim(lprogram), " End=",trim(ltoken(3))
                 endif 
              endif 
              goto 100
           endif                         
! Normal case
           if(num_token .eq. 2) then
              lerr_msg="ERROR: END section "//TRIM(lsection)//" without corresponding BEGIN"
           else
              lerr_msg="ERROR: END section "//TRIM(lsection)//" for station "&
     &             //trim(ltoken(3))//" without corresponding BEGIN"
           endif 
           ierr=11             
         endif
         if(Num_Token .lt. 2) then
            lerr_msg= "ERROR: END with no Section_Name specified!"
            ierr=12           
         endif
         call capitalize(ltoken(2))
         if(ltoken(2) .ne. lsection) then 
           lerr_msg="Mismatch between begin section = "//TRIM(lsection)//" and end section ="//ltoken(2)
           ierr=13
         endif
         if(lsection .eq. "STATION") then
            call capitalize(ltoken(3))
            if(lstation .ne. ltoken(3)) then
              lerr_msg="Station section mismatch between begin station "&
     &               //TRIM(lstation)//" and end station "//TRIM(ltoken(1))
              ierr=14               
            endif         
         endif
         if(lsection .eq. "PROCESS") lsection="HISTORY"
         kopen=.false. 
      case("DEFAULT_DIR")       
         if(ltoken(2)(1:1) .eq. "/") then
            ldefault_dir(ilevel)=ltoken(2)
         else
            ldefault_dir(ilevel)=trim(ldefault_dir(ilevel-1))//ltoken(2)
         endif 
         nch=LEN_TRIM(ldefault_dir(ilevel))
         if(ldefault_dir(ilevel)(nch:nch) .ne. "/") ldefault_dir(ilevel)(nch+1:nch+1)="/"
          
      case("CREATED","CREATEDBY","HISTORY") 
         if(.not.khistory) then 
            lerr_msg="KEYWORD "//TRIM(lkeyword)//" found in outside of HISTORY section!"
            ierr=30
          endif
      case("PROGRAM")
         continue            
      case("SESSION","ALTSESSIONID")
        if(lsection .ne. "SESSION") then
           lerr_msg="KEYWORD "//TRIM(lkeyword)//" found outside of SESSION section!"
           ierr=31
        endif

        if(lkeyword .eq. "SESSION")then
          lsession=ltoken(2)   
          call capitalize(lsession)   
        else
          lsessionalt=ltoken(2)
          call capitalize(lsessionalt)
        endif 

      case default
        if(khistory) goto 100     
        if(kprogram .and. .not. ksolve) goto 100
! all of the remaining kinds of keywords specify files
        if(num_token .gt. 4) then          
          lerr_msg="Too many tokens! "//Trim(ldum)
          ierr=33
          goto 120
        endif

        if(index(ldum, ".nc") .eq. 0) then
          lerr_msg="Expecting filename and none found: "//Trim(ldum)
          ierr=32
          goto 120
        endif
          
        lTimeTag=ltoken(num_token)
        call capitalize(lTimeTag)
        if(lTimeTag .eq. "IMPLICIT" .or. lTimeTag .eq. "EXPLICIT") then
           Num_token = Num_token -1
        else
          lTimeTag=" "
        endif

        lfilnam=ltoken(Num_Token)
        if(index(lfilnam,".nc")  .eq. 0) then
          lerr_msg="Expected filename: "//trim(ldum)
          ierr=34
        endif
          
!        write(*,*) "File :", trim(lfilnam)
        if(num_token .ge. 3) then
           lstub=ltoken(1)
           larray=ltoken(2)
        else if(num_token .eq. 2) then
           lstub=ltoken(1)
           larray=ltoken(1)
        else
           nch=index(lfilnam,"_") 
           if(nch .eq. 0) nch=index(lfilnam,".") 
             if(nch .eq. 0) then 
               ierr=35
               lerr_msg="Can't extract kind from filename: "//trim(lfilnam)
               goto 120
             endif
             lstub=lfilnam(1:nch-1)
             larray=lstub
          endif 

!          writE(*,*) "ldir: ",trim(ldefault_dir(ilevel))
!          write(*,*) "File: ",trim(lfilnam)       
        
          lfilnam_tot=TRIM(ldefault_dir(ilevel))//lfilnam 
          inquire(file=lfilnam_tot,exist=kexist)
          if(.not.kexist) then 
             ierr=20
             lerr_msg="File not found: "//TRIM(lfilnam_tot)
             goto 120
          else       
            if(kverbose) write(*,'(a)') "    ...found file "//TRIM(lfilnam_tot)//" Kind= " &
     &                //Trim(lstub)//" Array= "//trim(larray)
          endif 
 
          call capitalize(lstub)                           
          if(lstub .eq. "HEAD") then
             khead_found=.true.                             
          else                         
! Some other kind of file.  This is an error if we haven't opened the head yet.
            if(.not.khead_found) then
               ierr=60
               lerr_msg="In session "//trim(lsession)//" trying to open file before reading head." 
               return 
            endif
          endif 
! Extract the 'kind' if there is one
         call get_vgosDB_filename_arg(lfilnam,"k",lkind,ierr)
         call get_vgosDB_filename_arg(lfilnam,"b",lband,ierr)

! This is the file that we oen. 
         lnetCDF_file=lfilnam_tot
!         write(*,*) trim(lfilnam_tot)
         kverbose=.false.
         call read_netCDF_var_info(kverbose,lerr_msg,ierr)
!         write(*,*) "enter i"
!         read(*,*) i 

! Go through the entire list to see if we have a match.
         do i=1,num_list
!           write(*,*) ">> ", i, " ", vgosDB_list(i)%lname
           if(vgosDB_list(i)%kfound) goto 90  !already found.  Don't need to find again. 
           if(vgosDB_list(i)%lscope .ne. lsection) goto 90      !Scope does not match
           select case(lsection)
           case("OBSERVATION")
             if(vgosDB_list(i)%lband .ne. lband) goto 90        !band does not match
             ltemp=lband
           case("STATION")
             if(vgosDB_list(i)%lstation .ne. lstation) goto 90  !station does not batch
             ltemp=lstation
           case default
             ltemp=" "
           end select 
   
! Now see if such a variable exists in the file.      
           call get_netCDF_varid(vgosDB_list(i)%lname,ivarid,lerr_msg,ierr)     
           if(ivarid .eq. 0) goto 90                        ! no match                 
! Now see the size of the array.
! Fundamentally two different kinds. 1. One value per observation.  2. Many values per observation. 
           num_dim_in =num_dim_var(ivarid)      
  
! See if we are repeating this for all observations or scans. If so then last dimension is implicit.  
           if(ivar_repeat(ivarid) .ne. 0)  then                  
              vgosDB_list(i)%ndim=num_dim_in
              idim1=1
              do j=1,num_dim_in
                 vgosDB_list(i)%idim_size(j)=idim_size_var(ivar_dimid(j,ivarid))
                 idim1=idim1*vgosDB_list(i)%idim_size(j)
              end do
              idim2=ivar_repeat(ivarid)             
           else
             if(num_dim_in .eq. 1) then
                vgosDB_list(i)%ndim=1
                vgosDB_list(i)%idim_size(1)=1        
                idim1=1        
             else
                vgosDB_list(i)%ndim =num_dim_in-1
                idim1 =1
                do j=1,num_dim_in -1
                  vgosDB_list(i)%idim_size(j)=idim_size_var(ivar_dimid(j,ivarid))
                  idim1=idim1*vgosDB_list(i)%idim_size(j)
                end do 
             endif 
               idim2=idim_size_var(ivar_dimid(num_dim_in ,ivarid))
           endif 
           if(.not. kheader_line) then
             kheader_line=.true.
             write(*,'("Name",26x,a)') "Stat/Band       D1    D2    File"
           endif 
                              
          
           write(*,'(a30,1x,a,1x,2i6," in ",a)') vgosDB_list(i)%lname, ltemp, &    
     &           vgosDB_list(i)%idim_size(1),vgosDB_list(i)%idim_size(2), trim(lfilnam_tot)
!           goto 90 
  
! Found a match. Now must read it in.            
! General scheme:
! IF 1-dim then
!   Allocate space for 1-Dim
!   read in
!   If repeat attribute is <>0, then copy to rest of observations.
! Else
!   allocate space for 2-dim
!   read in
!   If repeat attribute is <>0, then copy to rest of the observations. 
! endif
           isize1=vgosDB_list(i)%idim_size(1)
           vgosDB_list(i)%itype=ivar_type(ivarid)
           vgosDB_list(i)%kfound=.true. 
           select case(ivar_type(ivarid))
           case(nf_double)     !real*8 
             allocate(vgosDB_list(i)%value8(idim1,idim2))
             iretval=nf_get_var_double(ncid,ivarid,vgosDB_list(i)%value8)                              
             if(iretval .ne. nf_noerr) call handle_netCDF_err(iretval)   
             if(ivar_repeat(ivarid) .ne. 0) then
               do j=2,ivar_repeat(ivarid)
                  vgosDB_list(i)%value8(1:idim1,j)=vgosDB_list(i)%value8(1:idim1,1)
               end do 
             endif                                            
           case(nf_byte)  !integer*1         
             allocate(vgosDB_list(i)%ivalue1(idim1,idim2))
             iretval=nf_get_var_int1(ncid,ivarid,vgosDB_list(i)%ivalue1)                              
             if(iretval .ne. nf_noerr) call handle_netCDF_err(iretval)   
             if(ivar_repeat(ivarid) .ne. 0) then
                do j=2,ivar_repeat(ivarid)
                  vgosDB_list(i)%ivalue1(1:idim1,j)=vgosDB_list(i)%ivalue1(1:idim1,1)
                end do 
             endif  
            
          case(nf_short)  !integer*2
            allocate(vgosDB_list(i)%ivalue2(idim1,idim2))
            iretval=nf_get_var_int2(ncid,ivarid,vgosDB_list(i)%ivalue2)                              
            if(iretval .ne. nf_noerr) call handle_netCDF_err(iretval)  
            if(ivar_repeat(ivarid) .ne. 0) then
               do j=2,ivar_repeat(ivarid)
                 vgosDB_list(i)%ivalue2(1:isize1,j)=vgosDB_list(i)%ivalue2(1:isize1,1)
               end do 
            endif              
          case(nf_int)  !integer*4
             allocate(vgosDB_list(i)%ivalue4(idim1,idim2))
            iretval=nf_get_var_int(ncid,ivarid,vgosDB_list(i)%ivalue4)                              
            if(iretval .ne. nf_noerr) call handle_netCDF_err(iretval)  
            if(ivar_repeat(ivarid) .ne. 0) then
               do j=2,ivar_repeat(ivarid)
                 vgosDB_list(i)%ivalue4(1:isize1,j)=vgosDB_list(i)%ivalue4(1:isize1,1)
               end do 
            endif              
          case(nf_char)     ! string        
             allocate(vgosDB_list(i)%string(idim1,idim2))
             iretval=nf_get_var_text(ncid,ivarid,vgosDB_list(i)%string)     
                     
             if(iretval .ne. nf_noerr) call handle_netCDF_err(iretval)   
             if(ivar_repeat(ivarid) .ne. 0) then
                 do j=2,ivar_repeat(ivarid)
                 vgosDB_list(i)%string(1:idim1,j)=vgosDB_list(i)%string(1:idim1,1)
                end do      
             endif       
          end select               
           
90         continue
         end do    
! close netcdf file that is opened with read_netCDF_var
         iretval=nf_close(ncid)
         if(iretval .ne. nf_noerr) call handle_NetCDF_err(iretval)       
        END SELECT KEYWORD
        goto 100 
        
120     continue
        if(ierr .ne. 0) then     
!           write(*,*) "Ierr: ", ierr 
!           write(*,*) "line 501: ",trim(ldum)     
           if(kverbose) write(*,*) "Exit Read_vgosDB error: ", ierr 
           close(lu_wrp)
           return
        endif


200     continue       
        close(lu_wrp)
        if(kverbose) write(*,*) "Exiting read_vgosDB without error!"
        return
        end    

